using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace System.IO
{
    public abstract class Flags : IDatatizable
    {
        protected bool[] _flags;

        public bool this[int index] { get { return _flags[index]; } set { _flags[index] = value; } }

        #region IDatatizable Members

        public abstract void Write(DataStream stream);

        #endregion

        protected long GetValue()
        {
            long data = 0;
            for (int i = 0; i < _flags.Length; i++) if (_flags[i]) data |= Convert.ToInt64(Math.Pow(2, i));
            return data;
        }

        protected static void FillBoolArray(long data, bool[] flags)
        {
            long compare = 0;
            for (int i = 0; i < flags.Length; i++)
            {
                compare = Convert.ToInt64(Math.Pow(2, i));
                flags[i] = ((data & compare) == compare);
            }
        }
    }

    public class Flags8 : Flags
    {
        public Flags8()
        { _flags = new bool[8]; }

        public Flags8(byte value)
            : this()
        {
            FillBoolArray(value, _flags);
        }

        internal Flags8(DataStream stream)
        { FillBoolArray(stream.ReadByte(), _flags); }

        public override void Write(DataStream stream)
        {
            stream.Write((byte)GetValue());
        }
    }

    public class Flags16 : Flags
    {
        public Flags16()
        { _flags = new bool[16]; }

        public Flags16(short value)
            : this()
        {
            FillBoolArray(value, _flags);
        }

        internal Flags16(DataStream stream)
        { FillBoolArray(stream.ReadInt16(), _flags); }

        public override void Write(DataStream stream)
        {
            stream.Write((short)GetValue());
        }
    }

    public class Flags32 : Flags
    {
        public Flags32()
        { _flags = new bool[32]; }

        public Flags32(int value)
            : this()
        {
            FillBoolArray(value, _flags);
        }

        internal Flags32(DataStream stream)
        { FillBoolArray(stream.ReadInt32(), _flags); }

        public override void Write(DataStream stream)
        {
            stream.Write((int)GetValue());
        }
    }

    public class Flags64 : Flags
    {
        public Flags64()
        { _flags = new bool[64]; }

        public Flags64(long value)
            : this()
        {
            FillBoolArray(value, _flags);
        }

        internal Flags64(DataStream stream)
        { FillBoolArray(stream.ReadInt64(), _flags); }

        public override void Write(DataStream stream)
        {
            stream.Write(GetValue());
        }
    }
}
