using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;

namespace System.IO
{
    public interface IDatatizable
    {
        void Write(DataStream stream);
    }

    [AttributeUsage(AttributeTargets.Class | AttributeTargets.Struct, Inherited = false, AllowMultiple = true)]
    public sealed class DatatizableAttribute : Attribute
    {
        readonly BindingFlags _bindingFlags;
        public BindingFlags PositionalString { get { return _bindingFlags; } }

        public DatatizableAttribute(BindingFlags bindingFlags)
        {
            _bindingFlags = bindingFlags;
        }
        
        public static BindingFlags GetBindingFlags(object value)
        {
            object[] datatizableAttributes = value.GetType().GetCustomAttributes(typeof(DatatizableAttribute), false);
            BindingFlags bf = BindingFlags.Default;
            for (int i = 0; i < datatizableAttributes.Length; i++)
                bf |= ((DatatizableAttribute)datatizableAttributes[i])._bindingFlags;
            return bf;
        }
    }

}
